
/*
   helimenu.c
   
   Menu and dialog routines for helicon.
   
   Author: Mark E. Nash
   Date:   June, 1988
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<malloc.h>
#include	<string.h>
#include	<conio.h>
#include	<graph.h>
#include	"helimacs.h"
#include	"helimess.h"
#include	"helstruc.h"
#include	"helfdef.h"


/* externals */
extern char msg[];			/* in heliglob.h */
extern unsigned delcon;			/* in heliglob.h */
extern char *dstr[];			/* in heliglob.h */
extern int dlocs[];			/* in heliglob.h */
extern int einfo[];			/* in heliglob.h */
extern int dlogs[];			/* in heliglob.h */
extern unsigned m1, m2, m3, m4;		/* in heliglob.h */



struct menutop *buildmenus()
/*
   buildmenus() builds the helicon menu data structure.
   
   Entry: This routine is called during initialization, but after video init.
   
   Exit:  If no errors, the menu data structure has been built, and returns
   		a pointer to the menu structure,
          Else returns NULL.
*/
	{
	struct menutop *mt;		/* pointer to top of menu structure */
	int	id;				/* for menu references */
	extern char menuset[];			/* in heliglob.h */
	extern char *flist[MAXMENUSIZE];	/* in heliglob.h */
	extern char *analist[MAXMENUSIZE];	/* in heliglob.h */
	extern char *fitlist[MAXMENUSIZE];	/* in heliglob.h */
	extern char *blist[MAXMENUSIZE];	/* in heliglob.h */
	extern char *dsplist[MAXMENUSIZE];	/* in heliglob.h */
	extern char *utilist[MAXMENUSIZE];	/* in heliglob.h */


	if ((mt = btopmenu(EMAXCOL, EMAXROW, MBCOLOR, MBICOLOR)) == NULL)
		{
		printf("\7");
		return(mt);
		}
	
/* put the menu set title in */
	mt->title = menuset;
	mt->tbcolor = TBCOLOR;
	mt->tcolor = TCOLOR;
	
/* add the file menu */
	if ((id = addmenu(mt, flist, FLSIZE, UICOLOR, UBCOLOR)) == MENUERR)
		{
		sprintf(msg, U_NOADD, flist[0]);
		err_window(msg);
		free_menu(&mt);
		return(mt);
		}

/* add the analyze menu */
	if ((id = addmenu(mt, analist, ANASIZE, UICOLOR, UBCOLOR)) == MENUERR)
		{
		sprintf(msg, U_NOADD, analist[0]);
		err_window(msg);
		free_menu(&mt);
		return(mt);
		}

/* add the fit menu */
	if ((id = addmenu(mt, fitlist, FITSIZE, UICOLOR, UBCOLOR)) == MENUERR)
		{
		sprintf(msg, U_NOADD, fitlist[0]);
		err_window(msg);
		free_menu(&mt);
		return(mt);
		}

/* add the build menu */
	if ((id = addmenu(mt, blist, BSIZE, UICOLOR, UBCOLOR)) == MENUERR)
		{
		sprintf(msg, U_NOADD, blist[0]);
		err_window(msg);
		free_menu(&mt);
		return(mt);
		}

/* add the display menu */
	if ((id = addmenu(mt, dsplist, DSPSIZE, UICOLOR, UBCOLOR)) == MENUERR)
		{
		sprintf(msg, U_NOADD, dsplist[0]);
		err_window(msg);
		free_menu(&mt);
		return(mt);
		}

/* add the utilities menu */
	if ((id = addmenu(mt, utilist, UTISIZE, UICOLOR, UBCOLOR)) == MENUERR)
		{
		sprintf(msg, U_NOADD, utilist[0]);
		err_window(msg);
		free_menu(&mt);
		return(mt);
		}

/* initialize any hilight flags */
	toghil(mt, ANAID, ASCALID);

	return(mt);
	}
	
	
	
struct menutop *btopmenu(mcols, mrows, bcolor, icolor)
	unsigned mcols, mrows;
	short	bcolor, icolor;
/*
   btopmenu() builds the top of the menu structure.
   
   Entry: This function is called first when building a menu structure.
          mcols is the last column number the menu can use.
	  mrows is the last row number the menu can use.
	  bcolor is the color of the command bar.
	  icolor is the color of the menu titles.
   
   Exit:  If no errors, returns the top of the menu structure,
          else returns NULL.
*/
	{
	struct menutop *mt;
	
	
	if ((mt = (struct menutop *)malloc(sizeof(struct menutop))) != NULL)
		{
/* update memory status */
		memupdt((unsigned long)sizeof(struct menutop), ALLOC);
		mt->maxcols = mcols;
		mt->maxrows = mrows;
		mt->barcolor = bcolor;
		mt->itemcolor = icolor;
		mt->start = NULL;
		}
	

	return(mt);
	}
	
	
	
int addmenu(top, marray, asize, micolor, mcolor)
	struct menutop *top;
	char *marray[MAXMENUSIZE];
	unsigned asize;
	short micolor, mcolor;
/*
   addmenu() adds a menu to the menu list.
   
   Entry: btopmenu() has been called successfully.
          top refers to the top of the menu structure.
	  marray contains the title in element 0 and the item list following.
	  asize is the size of the marray.
          micolor is the menu item color.
          mcolor is the background color of the menu.
	  
   Exit:  If no errors, returns the menu id,
          else returns MENUERR.
*/
	{
	int	mid;				/* menu id number */
	int	ctr;				/* jeep counter */
	struct menu *mp, *mr, *last;		/* pointers to a menu */
	int	x, y;				/* background image info */
	struct xycoord xy;			/* for background info */
	
	
/* exit if no menu top */
	if (top == NULL)
		return(MENUERR);

/* allocate storage */		
	if ((mp = (struct menu *)malloc(sizeof(struct menu))) == NULL)
		return(MENUERR);
/* update memory status */
	memupdt((unsigned long)sizeof(struct menu), ALLOC);
		
/* set title */
	mp->title = marray[0];
	
/* menu is not pulled down */
	mp->pd = FALSE;
	
/* find the bar position, get the last id and the last pointer */
	mr = top->start;
	mid = NULLID;
	last = mr;
	while (mr != NULL)
		{
		mid = mr->menuid;
		last = mr;
		mr = mr->next;
		}

	if (last == NULL)
		mp->barpos = BARSTART;
	else
		mp->barpos = strlen(last->title) + last->barpos + MISPACE;
	
	if (mp->barpos > (top->maxcols - strlen(marray[0])))
		return(MENUERR);
		
/* set the colors */
	mp->micolor = micolor;
	mp->menucolor = mcolor;
	
/* set the id */
	mid++;
	mp->menuid = mid;

/* set the menu title hilight box */
	mp->box.x1 = HPIXCHAR * mp->barpos - HPIXCHAR / 2;
	mp->box.y1 = MBY1;
	mp->box.x2 = mp->box.x1 + HPIXCHAR * (strlen(marray[0]) + 1);
	mp->box.y2 = MBY2;
	
/* null out the next pointer */
	mp->next = NULL;
	
/* now add it to the list */
	if (last == NULL)
		top->start = mp;
	else
		last->next = mp;

/* set the default menu width to the size of the menu title */
	mp->width = strlen(marray[0]);
	for (ctr=1; ctr<asize; ctr++)
/* update menu width if necessary */
		if (strlen(marray[ctr]) > mp->width)
			{
			mp->width = strlen(marray[ctr]) + 2;
			if (mp->width + mp->barpos > EMAXCOL)
				{
				sprintf(msg, U_BIGITEM, marray[ctr], 
								marray[0]);
				err_window(msg);
				return(MENUERR);
				}
			}

/* add items */
	mp->top = NULL;
	for (ctr=1; ctr<asize; ctr++)
		if (additem(mp, marray[ctr]) == MENUERR)
			{
			sprintf(msg, U_NOITEM, marray[ctr]);
			err_window(msg);
			return(MENUERR);
			}

/* get background information */
	if (asize > 1)
		{
	/* upper left */
		x = mp->box.x1;		/* tie this to the hilite box */
		y = MBY2 + 1;
		xy = _getlogcoord(x, y);
		mp->ix1 = xy.xcoord;
		mp->iy1 = xy.ycoord;
	/* lower right */
		x += HPIXCHAR * mp->width;
		y += VLINSIZE * (asize - 1);
		xy = _getlogcoord(x, y);
		mp->ix2 = xy.xcoord;
		mp->iy2 = xy.ycoord;

/* allocate memory for background */
		mp->size = _imagesize(mp->ix1, mp->iy1, mp->ix2, mp->iy2);
		if ((mp->background = (char far *)halloc(mp->size, 
							sizeof(char))) == NULL)
			{
			printf("\7");
			return(MENUERR);
			}
/* update memory status */
		memupdt((unsigned long)mp->size, ALLOC);
		}
	else
	/* no items */
		{
		mp->ix1 = 0;
		mp->iy1 = 0;
		mp->ix2 = 0;
		mp->iy2 = 0;
		mp->size = 0l;
		mp->background = NULL;
		}
	
	return(mid);
	}
	
	
	
int additem(mptr, item)
	struct menu *mptr;
	char *item;
/*
   additem() adds an item to a menu.
   
   Entry: mptr points to the menu to add the item to.
          item is the name of the item.
	  
   Exit:  If no errors, the item has been added to the menu and the item id
   		is returned,
	  else returns MENUERR.
*/
	{
	unsigned id;				/* item id */
	struct menuitem *mi, *ip, *last;	/* menu item */
	
	
/* allocate storage */
	if ((ip = (struct menuitem *)malloc(sizeof(struct menuitem)))
								== NULL)
		return(MENUERR);
/* update memory status */
	memupdt((unsigned long)sizeof(struct menuitem), ALLOC);

/* get last pointer and current id number */
	mi = mptr->top;
	id = NULLID;
	last = mi;
	while (mi != NULL)
		{
		id = mi->itemid;
		last = mi;
		mi = mi->next;
		}
		
/* put in item name, id, hilight default, and null pointer */
	id++;
	ip->entry = item;
	ip->itemid = id;
	ip->hf = FALSE;
	ip->next = NULL;

/* set the hilight box */
	ip->box.x1 = mptr->box.x1;
	ip->box.y1 = MBY1 + VLINSIZE * (id + 1);
 	ip->box.x2 = mptr->box.x1 + HPIXCHAR * mptr->width;
	ip->box.y2 = ip->box.y1 + VLINSIZE;
	
/* add it to the list */
	if (last == NULL)
		mptr->top = ip;
	else
		last->next = ip;

	return(id);
	}
	
	
	
int free_menu(top)
	struct menutop **top;
/*
   free_menu() liberates the menu structure.
   
   Entry: *top points to the menu structure.
   
   Exit:  *top points to NULL.
          If no errors, the list has been distroyed,
	  else returns ERR.
*/
	{
	struct menu *mp, *mtemp;		/* menu structure pointers */
	struct menuitem *mi, *itemp;		/* menu item pointers */
	
	
	if (*top == NULL)
		return(NOERR);
		
	mp = (*top)->start;
	while (mp != NULL)
		{
		mi = mp->top;
		while (mi != NULL)
			{
			itemp = mi;
			mi = mi->next;
			free((void *)itemp);
/* update memory status */
			memupdt((unsigned long)sizeof(struct menuitem), FRE);
			}
		hfree((void huge *)mp->background);
/* update memory status */
		memupdt((unsigned long)mp->size, FRE);
		mtemp = mp;
		mp = mp->next;
		free((void *)mtemp);
/* update memory status */
		memupdt((unsigned long)sizeof(struct menu), FRE);
		}
	
	free((void *)(*top));
/* update memory status */
	memupdt((unsigned long)sizeof(struct menutop), FRE);
	*top = NULL;
	
/* check the heap */
	switch (_heapchk())
		{
		case _HEAPOK:
		case _HEAPEMPTY:
			break;
		case _HEAPBADBEGIN:
		case _HEAPBADNODE:
			printf("\7");
			return(ERR);
		}
/* update memory status */
	memupdt((unsigned long)sizeof(struct menu), FRE);
	
	return(NOERR);
	}
	
	
	
int menubar(mt)
	struct menutop *mt;
/*
   menubar() displays the title, menu bar and menu titles.
   
   Entry: The menu structure has been built with no errors.
          mt points to the top of the menu structure.
   
   Exit:  If no errors, the menu bar has been displayed (EGA hi-res 
   		color graphics mode), and returns NOERR,
	  else returns ERR.
*/
	{
	unsigned ctr;			/* gp counter */
	struct menu *me;		/* menu pointer */
	struct xycoord xy1, xy2;	/* upper left and lower right log c. */
	
	
	if (mt == NULL)
		return(ERR);

/* turn off mouse cursor */
	MOUSE(2, 0, 0, 0);

/* draw the title bar */
	xy1 = _getlogcoord(TBX1, TBY1);
	xy2 = _getlogcoord(TBX2, TBY2);
	_setcolor(mt->tbcolor);
	if (_rectangle(_GFILLINTERIOR, xy1.xcoord, xy1.ycoord, xy2.xcoord, 
							xy2.ycoord) == 0)
		return(ERR);

/* draw the menu set title */
	eputs_ega(mt->title, _getlogcoord(MSTX, MSTY).xcoord, 
		    _getlogcoord(MSTX, MSTY).ycoord, WRLEN, mt->tcolor, TRUE);
		
/* draw the menu bar */
	xy1 = _getlogcoord(MBX1, MBY1);
	xy2 = _getlogcoord(MBX2, MBY2);
	_setcolor(mt->barcolor);
	if (_rectangle(_GFILLINTERIOR, xy1.xcoord, xy1.ycoord, xy2.xcoord, 
							xy2.ycoord) == 0)
		return(ERR);
		
/* now draw each menu title */
	me = mt->start;
	while (me != NULL)
		{
		eputs_ega(me->title, 
			_getlogcoord(HPIXCHAR * me->barpos + HPIXCHAR/2, 
					VLINSIZE + VPIXCHAR/2).xcoord,
			_getlogcoord(HPIXCHAR * me->barpos + HPIXCHAR/2, 
					VLINSIZE + VPIXCHAR/2).ycoord,
						WRLEN, mt->itemcolor, FALSE);
		me = me->next;
		}

/* turn mouse back on */
	MOUSE(1, 0, 0, 0);
		
	return(NOERR);
	}
	
	
	
int toghil(mt, idmenu, iditem)
	struct menutop *mt;
	unsigned idmenu, iditem;
/*
   toghil() toggles a menu item hiliter.
   
   Entry: mt points to the menu top.
          idmenu, iditem are the id's of the item to toggle.
	  
   Exit:  The next time the item is displayed on the menu, it's hilite will be
   		changed.
*/
	{
	struct menu *mp;		/* point to menu */
	struct menuitem *ip;		/* point to item */
	int	mfound, ifound;		/* flags to tell if found */
	
	
	if (mt == NULL)
		return(ERR);
		
	mfound = FALSE;
	ifound = FALSE;

/* get the menu */
	mp = mt->start;
	while (mp != NULL && !mfound)
		if (mp->menuid == idmenu)
			mfound = TRUE;
		else
			mp = mp->next;

/* get the item */
	if (mfound)
		{
		ip = mp->top;
		while (ip != NULL && !ifound)
			if (ip->itemid == iditem)
				ifound = TRUE;
			else
				ip = ip->next;
		}
		
/* toggle the item hilite flag */
	if (ifound)
		if (ip->hf == TRUE)
			ip->hf = FALSE;
		else
			ip->hf = TRUE;

	return(NOERR);
	}
	
	
	
int pulldown(mt, idmenu)
	struct menutop *mt;
	unsigned idmenu;
/*
   pulldown() pulls down a menu.
   
   Entry: mt points to the top of the menu structure.
          idmenu is the id of the menu to pull down.
	  
   Exit:  The menu idmenu has been pulled down.
          Returns NOERR, else returns ERR if errors occure.
*/
	{
	struct menu *mp;		/* point to menu */
	struct menuitem *ip;		/* point to item */
	register x, y;			/* for drawing items */
	
	
	if (mt == NULL)
		return(ERR);
	
/* get the menu */
	mp = mt->start;
	while (mp != NULL && mp->menuid != idmenu)
		mp = mp->next;
			
	if (mp == NULL)
		return(ERR);
		
/* get out if menu already pulled down or nothing to pull down */
	if (mp->pd == TRUE || mp->size == 0l)
		return(NOERR);

/* turn off mouse cursor */
	MOUSE(2, 0, 0, 0);

/* save the background */
	_getimage(mp->ix1, mp->iy1, mp->ix2, mp->iy2, mp->background);

/* draw the box */
	_setcolor(mp->menucolor);
	_rectangle(_GFILLINTERIOR, mp->ix1, mp->iy1, mp->ix2, mp->iy2);
	
/* draw the menu items */
	ip = mp->top;
	x = mp->ix1 + HPIXCHAR / 2;
	y = mp->iy1 + VPIXCHAR / 2;
	while (ip != NULL)
		{
		msg[0] = '\0';
		if (ip->hf == TRUE)
			strcat(msg, "*");	/* hilite area */
		else
			strcat(msg, " ");
		strncat(msg, ip->entry, MESSLEN - 1);
		eputs_ega(msg, x, y, mp->width, mp->micolor, FALSE);
		y += VLINSIZE;
		ip = ip->next;
		}
		
/* report that the menu is pulled down */
	mp->pd = TRUE;

/* turn mouse back on */
	MOUSE(1, 0, 0, 0);
		
	return(NOERR);
	}
	
	
	
int letgo(mt, idmenu)
	struct menutop *mt;
	unsigned idmenu;
/*
   letgo() gets rid of a menu.
   
   Entry: mt points to the top of the menu structure.
          idmenu is the id of the menu to let go of.
	  
   Exit:  The menu idmenu has been let go.
          Returns NOERR, else returns ERR if errors occure.
*/
	{
	struct menu *mp;		/* point to menu */
	struct menuitem *ip;		/* point to item */
	register x1, y1;		/* for drawing items */
	
	
	if (mt == NULL)
		return(ERR);
	
/* get the menu */
	mp = mt->start;
	while (mp != NULL && mp->menuid != idmenu)
		mp = mp->next;
			
	if (mp == NULL)
		return(ERR);

/* turn off mouse cursor */
	MOUSE(2, 0, 0, 0);

/* restore the background */
	_putimage(mp->ix1, mp->iy1, mp->background, _GPSET);
	
/* report that this menu is no longer pulled down */
	mp->pd = FALSE;
	
/* turn mouse back on */
	MOUSE(1, 0, 0, 0);
		
	return(NOERR);
	}
	
	
	
int hilsel(mt, idmenu, iditem, blink)
	struct menutop *mt;
	unsigned idmenu, iditem, blink;
/*
   hilsel() hilights a menu selection.
   
   Entry: mt points to the top of the menu structure.
          idmenu is the menu id to hilight.
	  iditem is the item in idmenu to hilight (NULLID if none).
	  blink is TRUE if blinking is requested (blinks BLCYCLE times).

   Exit:  Returns NOERR if no errors, else ERR.
*/
	{
	struct menu *mp;		/* point to a menu */
	struct menuitem *ip;		/* point to an item */
	register xctr, yctr;		/* hilite box */
	int	x1, y1, x2, y2;		/* hilight box */
	int	cyc, cycmax;		/* cycle counter */
	unsigned delctr1, delctr2;	/* blink delay counter */
	
	
	if (mt == NULL)
		return(ERR);
	
/* get the menu */
	mp = mt->start;
	while (mp != NULL && mp->menuid != idmenu)
		mp = mp->next;
			
	if (mp == NULL)
		return(ERR);

	x1 = mp->box.x1;
	y1 = mp->box.y1;
	x2 = mp->box.x2;
	y2 = mp->box.y2;
	
/* get the item if there is one */
	if (iditem != NULLID)
		{
		ip = mp->top;
		while (ip != NULL && ip->itemid != iditem)
			ip = ip->next;

		if (ip == NULL)
			return(ERR);
			
		x1 = ip->box.x1;
		y1 = ip->box.y1;
		x2 = ip->box.x2;
		y2 = ip->box.y2;
		}
		
/* if blink, cycle this part */
	if (blink)
		cycmax = BLCYCLE;
	else
		cycmax = 1;
		
/* turn off mouse cursor */
	MOUSE(2, 0, 0, 0);

/* set up the ega registers */
	egafunc(BRIGHTWHITE, XOR);
		
	for (yctr=y1; yctr<=y2; yctr++)
		egaline(x1, x2, yctr);
/* blinking */
	for (cyc=1; cyc < cycmax; cyc++)
		{
/* delay loop */
#pragma loop_opt(off)
		for (delctr2=0; delctr2<BLCMAX; delctr2++)
			for (delctr1=0; delctr1<delcon; delctr1++);
#pragma loop_opt()
		for (yctr=y1; yctr<=y2; yctr++)
			egaline(x1, x2, yctr);
		}
		
	egareset();

/* turn mouse back on */
	MOUSE(1, 0, 0, 0);
		
	return(NOERR);
	}
	
	
	
struct dialog *buildlogs()
/*
   buildlogs() builds the helicon dialog data structure.
   
   Entry: This routine is called during initialization, but after video init.
   
   Exit:  If no errors, the dialog data structure has been built, and returns
   		a pointer to the dialog structure,
		also, the eboxes are associated with global pointers.
          Else returns NULL.
*/
	{
	struct dialog *top;		/* top of dialog list */
	struct dialog *curlog;		/* current dialog */
	int	curdt;			/* current index into dstr[] */
	int	curdloc;		/* current index into dlocs[] */
	int	curebox;		/* current index into einfo[] */
	struct xycoord xy;		/* to convert to log coords */
	int	xp, yp;			/* physical coordinates */
	int	id;			/* dialog id */
	int	dctr;			/* dialog counter */


/* initialize */
	id = NULLID;
	top = NULL;
	dctr = 0;			/* first dialog */
	curdt = 0;			/* init index for dstr */
	curdloc = 0;			/* init location data pointer */
	curebox = 0;			/* init index for einfo */

/* see if there are any dialogs */
	if (dlogs[dctr] == ENDLOG)
		return(NULL);
		
/* allocate storage */
	if ((top = (struct dialog *)malloc(sizeof(struct dialog))) == NULL)
		{
		printf("\7");
		return(top);
		}
/* update memory status */
	memupdt((unsigned long)sizeof(struct dialog), ALLOC);

/* init first one */
	curlog = top;
	id++;

	curlog->id = id;
	xp = dlogs[dctr];
	dctr++;
	yp = dlogs[dctr];
	dctr++;
	xy = _getlogcoord(xp, yp);
	curlog->dlx1 = xy.xcoord;
	curlog->dly1 = xy.ycoord;

	curlog->dcols = dlogs[dctr];
	dctr++;
	curlog->drows = dlogs[dctr];
	getds(curlog, &curdt, &curdloc);       /* get dialog strings */
	getebox(curlog, &curebox);	/* get dialog eboxes */
	dctr++;			/* point to next dialog */

/* get the rest of the dialogs */
	while (dlogs[dctr] != ENDLOG)
		{
/* allocate storage */
		if ((curlog->next = (struct dialog *)malloc(sizeof(
						struct dialog))) == NULL)
			{
			printf("\7");
			return(top);
			}
/* update memory status */
		memupdt((unsigned long)sizeof(struct dialog), ALLOC);
			
		curlog = curlog->next;
		id++;
		curlog->id = id;
		
		xp = dlogs[dctr];
		dctr++;
		yp = dlogs[dctr];
		dctr++;
		xy = _getlogcoord(xp, yp);
		curlog->dlx1 = xy.xcoord;
		curlog->dly1 = xy.ycoord;

		curlog->dcols = dlogs[dctr];
		dctr++;
		curlog->drows = dlogs[dctr];
		getds(curlog, &curdt, &curdloc);       /* get dialog strings */
		getebox(curlog, &curebox);	/* get dialog eboxes */
		dctr++;				/* point to next dialog */
		}
	
/* associate appropriate globals with eboxes */
	
	return(top);
	}
	
	
	
int getds(dl, cdt, cdl)
	struct dialog *dl;
	int *cdt, *cdl;
/*
   getds() gets the dialog strings for this dialog.
   
   Entry: dl points to the current dialog.
          cdt points to the index of the current dialog string in dstr[].
	  cdl points to the index of the current string location in dlocs[].
   
   Exit:  The strings for this dialog have been added to the structure.
          *cdt and *cdl have been updated.
*/
	{
	struct diatext *curdt;		/* point to a dialog text */
	

/* see if any strings are to be added */
	if (strcmp(dstr[*cdt], "_ENDDIALOG") != 0)
		{
/* get the first dialog string */
		if ((dl->dtext = (struct diatext *)malloc(sizeof(
						struct diatext))) == NULL)
			{
			printf("\7");
			return(ERR);
			}
/* update memory status */
		memupdt((unsigned long)sizeof(struct diatext), ALLOC);
			
		curdt = dl->dtext;
		curdt->next = NULL;
		curdt->text = dstr[*cdt];
		curdt->col = dlocs[*cdl];
		(*cdl)++;
		curdt->row = dlocs[*cdl];
		(*cdl)++;

/* get the rest of the dialog strings */
		(*cdt)++;	/* inc the index */
		while (strcmp(dstr[*cdt], "_ENDDIALOG") != 0)
			{
			if ((curdt->next = (struct diatext *)malloc(sizeof(
						struct diatext))) == NULL)
				{
				printf("\7");
				return(ERR);
				}
/* update memory status */
			memupdt((unsigned long)sizeof(struct diatext), ALLOC);
			
			curdt = curdt->next;
			curdt->next = NULL;
			curdt->text = dstr[*cdt];
			curdt->col = dlocs[*cdl];
			(*cdl)++;
			curdt->row = dlocs[*cdl];
			(*cdl)++;

			(*cdt)++;	/* inc the index */
			}
		}

/* get past the null string delimiter */
	(*cdt)++;
	
	return(NOERR);
	}
	
	
	
int getebox(dl, ceb)
	struct dialog *dl;
	int *ceb;
/*
   getebox() gets the dialog eboxes for this dialog.
   
   Entry: dl points to the current dialog.
          ceb is the current index for the einfo global.
	  The global tbxctr is current.
   
   Exit:  The eboxes for this dialog have been added to the structure.
          ceb has been updated.
*/
	{
	struct editbox *cureb;		/* point to a dialog text */
	int	px, py;			/* physical coordinates */
	struct xycoord xy;		/* for coord conversion */
	extern char tboxes[NUMTBXS][TBOXSIZE+1];	/* in heliglob.h */
	extern int tbxctr;		/* in heliglob.h */
	
	
/* see if any eboxes are to be added */
	if (einfo[*ceb] == DELIMITER);
	else
		{
/* get the first edit box */
		if ((dl->ebox = (struct editbox *)malloc(sizeof(
						struct editbox))) == NULL)
			{
			printf("\7");
			return(ERR);
			}
/* update memory status */
		memupdt((unsigned long)sizeof(struct editbox), ALLOC);

		cureb = dl->ebox;
		cureb->next = NULL;
		cureb->ebw = NULL;
		cureb->tbox = tboxes[tbxctr++];
		cureb->id = einfo[*ceb];
		(*ceb)++;
		px = einfo[*ceb];
		(*ceb)++;
		py = einfo[*ceb];
		(*ceb)++;
		xy = _getlogcoord(px, py);
		cureb->lx1 = xy.xcoord;
		cureb->ly1 = xy.ycoord;
		cureb->size = einfo[*ceb];
		(*ceb)++;		/* inc the index */

/* get the rest of the edit boxes */
		while (einfo[*ceb] != DELIMITER)
			{
			if ((cureb->next = (struct editbox *)malloc(sizeof(
						struct editbox))) == NULL)
				{
				printf("\7");
				return(ERR);
				}
/* update memory status */
			memupdt((unsigned long)sizeof(struct editbox), ALLOC);

			cureb = cureb->next;
			cureb->next = NULL;
			cureb->ebw = NULL;
			cureb->tbox = tboxes[tbxctr++];
			cureb->id = einfo[*ceb];
			(*ceb)++;
			px = einfo[*ceb];
			(*ceb)++;
			py = einfo[*ceb];
			(*ceb)++;
			xy = _getlogcoord(px, py);
			cureb->lx1 = xy.xcoord;
			cureb->ly1 = xy.ycoord;
			cureb->size = einfo[*ceb];
			(*ceb)++;		/* inc the index */
			}
		}

/* get past delimiter */
	(*ceb)++;

	return(NOERR);
	}



int free_logs(top)
	struct dialog **top;
/*
   free_logs() liberates the dialog list.
   
   Entry: *top points to the top of the list.
	  
   Exit:  *top points to NULL.
          If no errors, the list has been distroyed,
          else returns ERR.
*/
	{
	struct dialog *dp, *temp;		/* dialog pointers */
	struct diatext *dt, *dtemp;		/* part of dialog struct */
	struct editbox *eb, *etemp;		/* part of dialog struct */
	
	
	if (*top == NULL)
		return(NOERR);
		
	dp = *top;
	while (dp != NULL)
		{
/* free the texts */
		dt = dp->dtext;
		while (dt != NULL)
			{
			dtemp = dt;
			dt = dt->next;
			free((void *)dtemp);
/* update memory status */
			memupdt((unsigned long)sizeof(struct diatext), FRE);
			}
/* free the eboxes */
		eb = dp->ebox;
		while (eb != NULL)
			{
			etemp = eb;
			eb = eb->next;
			free((void *)etemp);
/* update memory status */
			memupdt((unsigned long)sizeof(struct editbox), FRE);
			}
/* free the dialog */
		temp = dp;
		dp = dp->next;
		free((void *)temp);
/* update memory status */
		memupdt((unsigned long)sizeof(struct dialog), FRE);
		}
		
	free((void *)(*top));
/* update memory status */
	memupdt((unsigned long)sizeof(struct dialog), FRE);
	*top = NULL;

/* check the heap */
		switch (_heapchk())
			{
			case _HEAPOK:
			case _HEAPEMPTY:
				break;
			case _HEAPBADBEGIN:
			case _HEAPBADNODE:
				printf("\7");
				return(ERR);
			}
	return(NOERR);
	}
